// R2BView.cpp : implementation of the CRtf2BmpTestView class
//

#include "stdafx.h"
#include "R2BTest.h"

#include "R2BDoc.h"
#include "R2BView.h"

// #include "R2BCom.h"

//-----------------------------------------------------------------
// DLL Interface Functions
extern "C" BOOL WINAPI RTSetPageTerm(char *, int);
extern "C" BOOL WINAPI RTInitRtf2Bmp(BOOL);
extern "C" BOOL WINAPI RTCloseRtf2Bmp(VOID);
extern "C" BOOL WINAPI RTLoadRtfFile(char *);
extern "C" BOOL WINAPI RTConvertRtf2Bmp(VOID);

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRtf2BmpTestView

IMPLEMENT_DYNCREATE(CRtf2BmpTestView, CView)

BEGIN_MESSAGE_MAP(CRtf2BmpTestView, CView)
  //{{AFX_MSG_MAP(CRtf2BmpTestView)
  ON_COMMAND(IDM_SCALE_1X, OnScale1x)
  ON_UPDATE_COMMAND_UI(IDM_SCALE_1X, OnUpdateScale1x)
  ON_COMMAND(IDM_SCALE_HALF, OnScaleHalf)
  ON_UPDATE_COMMAND_UI(IDM_SCALE_HALF, OnUpdateScaleHalf)
  ON_COMMAND(IDM_SCALE_QUARTER, OnScaleQuarter)
  ON_UPDATE_COMMAND_UI(IDM_SCALE_QUARTER, OnUpdateScaleQuarter)
  ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
  ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
  ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
  ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
  ON_WM_DESTROY()
  //}}AFX_MSG_MAP
  // Standard printing commands
  ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
  ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
  ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRtf2BmpTestView construction/destruction

CRtf2BmpTestView::CRtf2BmpTestView()
{
  // Set To The Group 3 Fax Fixed Width And Height, 
  m_nWidth  = 1728;
  m_nHeight = (int) ((11.5 / 8.5) * (float) m_nWidth);

  // Set Size For Whole Image
  m_nByteWidth       = m_nWidth / 8;
  m_nTotalBufferSize = m_nByteWidth * m_nHeight;

  // Create Memory Buffer To Hold Bytes That We Read In
  m_pBuffer = new char [m_nTotalBufferSize];
  char *pBuffer = m_pBuffer;
  for (int i=0; i < m_nTotalBufferSize; i++)
    *(pBuffer++) = (unsigned char) 0xFF;

  // Set Initial Scale Factor To Full Size
  m_nScale = 1;
}

CRtf2BmpTestView::~CRtf2BmpTestView()
{
  // Cleanup
  if (m_pBuffer != NULL) delete m_pBuffer;
}

BOOL CRtf2BmpTestView::PreCreateWindow(CREATESTRUCT& cs)
{
  // TODO: Modify the Window class or styles here by modifying
  //  the CREATESTRUCT cs

  return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CRtf2BmpTestView drawing

void CRtf2BmpTestView::OnDraw(CDC* pDC)
{
  CRtf2BmpTestDoc* pDoc = GetDocument();
  ASSERT_VALID(pDoc);
  
  // Create BITMAPINFO Struct
  // Color Table Goes At The End Of The BITMAPINFOHEADER Struct
  int nCU = 2;
  int nBMPISize = (sizeof (BITMAPINFOHEADER)) + (nCU * sizeof(RGBQUAD));
  char *pBMPI   = (char *)::malloc(nBMPISize);
  if (pBMPI == NULL) return;

  // Fill The BITMAPINFOHEADER Struct For Mono Bitmap
  memset(pBMPI, 0, nBMPISize);
  BITMAPINFOHEADER *pBMPH = (BITMAPINFOHEADER *) pBMPI;
  pBMPH->biSize        = sizeof (BITMAPINFOHEADER);
  pBMPH->biPlanes      = 1; 
  pBMPH->biBitCount    = 1; 
  pBMPH->biCompression = BI_RGB; 
  pBMPH->biSizeImage   = m_nTotalBufferSize; 
  pBMPH->biClrUsed     = 1 << pBMPH->biBitCount; 
  pBMPH->biWidth       = m_nWidth; 
  pBMPH->biHeight      = m_nHeight; 

  // Set The Colors To White And Black
  RGBQUAD *pRGB = (RGBQUAD *) (pBMPI + sizeof (BITMAPINFOHEADER));
  pRGB->rgbBlue = (BYTE) 0x00; pRGB->rgbRed = (BYTE) 0x00; pRGB->rgbGreen = (BYTE) 0x00;
  pRGB++;
  pRGB->rgbBlue = (BYTE) 0xFF; pRGB->rgbRed = (BYTE) 0xFF; pRGB->rgbGreen = (BYTE) 0xFF;

  // Create New Bitmap
  CBitmap bmp;
  bmp.CreateBitmap(m_nWidth, m_nHeight, 1, 1, NULL);

  // Copy Data Read In From File To Bitmap
  bmp.SetBitmapBits(m_nTotalBufferSize, m_pBuffer);

  // Create Compatible DC
  CDC dcCompat;
  dcCompat.CreateCompatibleDC(pDC);

  // Select Bitmap Into DC
  CBitmap* pOld = dcCompat.SelectObject(&bmp);

  // Copy Bitmap From Memory DC To Display
  pDC->StretchBlt(0, 0, m_nWidth/m_nScale, m_nHeight/m_nScale, &dcCompat, 
                  0, 0, m_nWidth,          m_nHeight,          SRCCOPY);

  // Restore
  dcCompat.SelectObject(pOld);

  // Clean Up 
  if (pBMPI != NULL) ::free(pBMPI);
}

/////////////////////////////////////////////////////////////////////////////
// CRtf2BmpTestView printing

BOOL CRtf2BmpTestView::OnPreparePrinting(CPrintInfo* pInfo)
{
  // default preparation
  return DoPreparePrinting(pInfo);
}

void CRtf2BmpTestView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
  // TODO: add extra initialization before printing
}

void CRtf2BmpTestView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
  // TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CRtf2BmpTestView diagnostics

#ifdef _DEBUG
void CRtf2BmpTestView::AssertValid() const
{
  CView::AssertValid();
}

void CRtf2BmpTestView::Dump(CDumpContext& dc) const
{
  CView::Dump(dc);
}

CRtf2BmpTestDoc* CRtf2BmpTestView::GetDocument() // non-debug version is inline
{
  ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CRtf2BmpTestDoc)));
  return (CRtf2BmpTestDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRtf2BmpTestView helper functions

void CRtf2BmpTestView::SetVersion() 
{
  // Determin OS Version
  OSVERSIONINFO osvi;
  memset(&osvi, 0, sizeof(OSVERSIONINFO));
  osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&osvi);

  // Set OS Version
  m_dwVersion = osvi.dwPlatformId;
  
  // Check OS Version
  CString strVersion;
  if (osvi.dwPlatformId == VER_PLATFORM_WIN32s)
    {
    strVersion.Format("Microsoft Win32s %d.%d (Build %d)",
                      osvi.dwMajorVersion,
                      osvi.dwMinorVersion,
                      osvi.dwBuildNumber & 0xFFFF);
    }
  else if (osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
    {
    strVersion.Format("Microsoft Windows 95 %d.%d (Build %d)",
                      osvi.dwMajorVersion,
                      osvi.dwMinorVersion,
                      osvi.dwBuildNumber & 0xFFFF);
    }
  else if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
    strVersion.Format("Microsoft Windows NT %d.%d (Build %d)",
                      osvi.dwMajorVersion,
                      osvi.dwMinorVersion,
                      osvi.dwBuildNumber & 0xFFFF);
    }

//  AfxMessageBox(strVersion);
  }

/////////////////////////////////////////////////////////////////////////////
// CRtf2BmpTestView message handlers

void CRtf2BmpTestView::OnInitialUpdate() 
{
  CView::OnInitialUpdate();

  // Determine Win95 or WinNT
  SetVersion();
  
  // Initializing conversion DLL
  RTInitRtf2Bmp(TRUE);

  // Make Sure Main Window Is On Top
  SetForegroundWindow();
}

void CRtf2BmpTestView::OnDestroy() 
{
  CView::OnDestroy();

  // Closing conversion DLL
  RTCloseRtf2Bmp();  
    
}

void CRtf2BmpTestView::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(FALSE);
}

void CRtf2BmpTestView::OnFileOpen() 
{
  // Create File Open Dialog
  CFileDialog dlg(TRUE, "bmp", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                  "RTF Files (*.rtf) | *.rtf");

  // Display, Then If OK Selected, Convert RTF File To A Raw Bitmap
  if (dlg.DoModal() == IDOK)
    {
    // Loading file for conversion
    RTLoadRtfFile(dlg.GetFileName().GetBuffer(1));
    
    // Converting file
    RTConvertRtf2Bmp();
    
    // Check To See If Memory Buffer Was Created
    if (m_pBuffer == NULL) return;
    
    // Set Output File Name XXXXXX.rtf --> XXXXXX.r2b
    char szTmpName[MAX_PATH];
    lstrcpy(szTmpName, dlg.GetFileName().GetBuffer(1));
    char *pLoc = strstr(szTmpName, ".rtf");
    if (pLoc != NULL) *pLoc = 0;
    lstrcat(szTmpName, ".r2b");

    HANDLE fh1 = ::CreateFile(szTmpName, GENERIC_READ | GENERIC_WRITE, 
                   (DWORD) 0, (LPSECURITY_ATTRIBUTES) NULL, 
                   OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 
                   (HANDLE) NULL);  
    DWORD dwTmp;
    ::ReadFile(fh1, (LPVOID) m_pBuffer, m_nTotalBufferSize,
              (LPDWORD) &dwTmp, (LPOVERLAPPED) NULL);
    ::CloseHandle(fh1);
    
    // Invert All Bits If Windows 95
    if (m_dwVersion == VER_PLATFORM_WIN32_WINDOWS)
      {
      char *pBuffer = m_pBuffer;
      for (int i=0; i < m_nTotalBufferSize; i++)
        *(pBuffer++) = (unsigned char) 0xFF - (unsigned char) *(m_pBuffer+i);
      }

    Invalidate();
    }
}

void CRtf2BmpTestView::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(FALSE);
}

void CRtf2BmpTestView::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(FALSE);
}

void CRtf2BmpTestView::OnScale1x() 
{
  m_nScale = 1;

  Invalidate();
}

void CRtf2BmpTestView::OnUpdateScale1x(CCmdUI* pCmdUI) 
{
  if (m_nScale == 1) pCmdUI->SetCheck(TRUE);
  else               pCmdUI->SetCheck(FALSE);
}

void CRtf2BmpTestView::OnScaleHalf() 
{
  m_nScale = 2;

  Invalidate();
}

void CRtf2BmpTestView::OnUpdateScaleHalf(CCmdUI* pCmdUI) 
{
  if (m_nScale == 2) pCmdUI->SetCheck(TRUE);
  else               pCmdUI->SetCheck(FALSE);
}

void CRtf2BmpTestView::OnScaleQuarter() 
{
  m_nScale = 4;

  Invalidate();
}

void CRtf2BmpTestView::OnUpdateScaleQuarter(CCmdUI* pCmdUI) 
{
  if (m_nScale == 4) pCmdUI->SetCheck(TRUE);
  else               pCmdUI->SetCheck(FALSE);
}

