//------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "ALabel.h"
#include <commctrl.h>
#include <shellapi.h>
#include <mmsystem.h>
#include <stdlib.h>

//------------------------------------------------------------------
#pragma package(smart_init)

__fastcall TActiveLabel::TActiveLabel(TComponent* owner) :
TActiveLabel_PARENT(owner)
{
  // Implementation.
  m_Tracking = false;

  // Properties.

  // Inherited.
  Caption = "ActiveLabel";
  Left    = 10;
  Top     = 10;
  Width   = 100;
  Height  = 32;

  // Local.
  m_Command       = "";
  m_AutoSize      = true;
  m_ActiveColor   = clRed;
  m_InactiveColor = clBlack;

  m_EnterSound    = "";
  m_CommandSound  = "";

}

// virtual
__fastcall TActiveLabel::~TActiveLabel()
{
}

// virtual
void __fastcall TActiveLabel::CreateParams(TCreateParams& params)
{
  TActiveLabel_PARENT::CreateParams(params);

  // We want a transparent window!
  params.ExStyle |= WS_EX_TRANSPARENT;
}

// virtual
void __fastcall TActiveLabel::Paint()
{
  Canvas->Brush->Style = bsClear;

  Canvas->Font->Assign(Font);
  Canvas->Font->Color = (m_Tracking) ? ActiveColor : InactiveColor;

  Canvas->TextOut(0, 0, m_Caption);
}

void __fastcall TActiveLabel::OnMouseMove(TMessage& msg)
{
  if(!(m_Tracking))
  {
    TRACKMOUSEEVENT info;

    info.cbSize       = sizeof(TRACKMOUSEEVENT);
    info.dwFlags      = TME_LEAVE;  // sends WM_MOUSELEAVE
    info.hwndTrack    = Handle;
    info.dwHoverTime  = 0;          // ignored

    // Use this version of TrackMouseEvent as it is supported on
    // all Win32 platforms. For Win98 & NT it is in USER32.DLL and
    // on Win95 it is in COMCTL32.DLL (>=4.70)
    _TrackMouseEvent(&info);

    m_Tracking = true;

    if(m_EnterSound != "")
    {
      PlaySound(m_EnterSound.c_str(), NULL,
         SND_FILENAME | SND_NOWAIT | SND_ASYNC);
    }

    Paint();
  }

  msg.Result = 0;
}

void __fastcall TActiveLabel::OnMouseLeave(TMessage& msg)
{
  // The TrackMouseEvent is automatically cancelled.
  m_Tracking = false;

  Paint();

  msg.Result = 0;
}

void __fastcall TActiveLabel::OnLButtonUp(TMessage& msg)
{
  msg.Result = 0;

  if(m_Command != "")
  {
    if(m_CommandSound != "")
    {
      PlaySound(m_CommandSound.c_str(), NULL,
         SND_FILENAME | SND_SYNC);
    }

    // Is this a special command?
    if(OnProcessSpecialCommand())
    {
      return;
    }

    // Not special command, just shell.
    Shell(m_Command);
  }
}

void __fastcall TActiveLabel::CalculateBounds()
{
  int textWidth;
  int textHeight;

  Canvas->Font->Assign(Font);

  textWidth = Canvas->TextWidth(Caption);
  textHeight = Canvas->TextHeight(Caption);

  Width = textWidth;
  Height = textHeight;
}

void __fastcall TActiveLabel::SetCommand(AnsiString command)
{
  m_Command = command;
}

void __fastcall TActiveLabel::SetAutoSize(bool autoSize)
{
  if(m_AutoSize != autoSize)
  {
    m_AutoSize = autoSize;

    CalculateBounds();
  }  
}

void __fastcall TActiveLabel::SetActiveColor(TColor color)
{
  if(m_ActiveColor != color)
  {
    m_ActiveColor = color;

    if(m_Tracking)
    {
      Paint();
    }
  }
}

void __fastcall TActiveLabel::SetInactiveColor(TColor color)
{
  if(m_InactiveColor != color)
  {
    m_InactiveColor = color;

    if(!(m_Tracking))
    {
      Paint();
    }
  }
}

void __fastcall TActiveLabel::SetCaption(AnsiString caption)
{
  if(m_Caption != caption)
  {
    m_Caption = caption;

    if(m_AutoSize)
    {
      CalculateBounds();

      Paint();
    }
  }
}

void __fastcall TActiveLabel::SetEnterSound(AnsiString enterSound)
{
  m_EnterSound = enterSound;
}

void __fastcall TActiveLabel::SetCommandSound(
   AnsiString commandSound)
{
  m_CommandSound = commandSound;
}

bool __fastcall TActiveLabel::OnProcessSpecialCommand()
{
  if(m_Command == "Exit")
  {
    OnExit();

    return(true);
  }

  if(m_Command == "Explore")
  {
    OnExplore();

    return(true);
  }

  return(false);
}      

void __fastcall TActiveLabel::OnExit()
{
  Application->Terminate();
}

void __fastcall TActiveLabel::OnExplore()
{
  TCHAR path[_MAX_PATH];

  if(0 != ::GetModuleFileName(HInstance, path, _MAX_PATH))
  {
    TCHAR drive[_MAX_DRIVE];
    TCHAR dir[_MAX_DIR];
    TCHAR name[_MAX_FNAME];
    TCHAR ext[_MAX_EXT];

    ::_splitpath(path, drive, dir, name, ext);

    // Open the drive in the Windows Explorer.
    Shell(drive);
  }
}

void __fastcall TActiveLabel::Shell(AnsiString cmd)
{
  ::ShellExecute(Handle,
    "open",
    cmd.c_str(),
    "",
    "",
    SW_SHOWNORMAL);
}


