//------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "CDIni.h"
#include "ALabel.h"
#include "MFrame.h"
#include <inifiles.hpp>
#include <mmsystem.h>

//------------------------------------------------------------------
#pragma package(smart_init)

__fastcall TCDIni::TCDIni()
{
   // Main variables.
   m_Title                   = "";
   m_Icon                     = "";
   m_Background            = "";
   m_Width                   = 0;
   m_Height                  = 0;
   m_StartSound            = "";
   m_ExitSound             = "";

   // Shell variables.
   m_Count                   = 0;
   m_Align                   = 0;
   m_LeftMargin            = 0;
   m_TopMargin             = 0;
   m_Spacing                = 0;
   m_ActiveColor          = 0;
   m_InactiveColor       = 0;
   m_ActiveSound          = "";
   m_ShellSound            = "";
   m_FontName               = "";
   m_FontSize               = 0;
}

__fastcall TCDIni::~TCDIni()
{
}

void __fastcall TCDIni::Initialize()
{
   AnsiString iniPathname;

   // First we figure out the current locale. On the CD we expect to
   // find a "DATA" directory that contains multiple CD.INI files
   // for each lang that is supported. There must be a *default*
   // CD.INI for use when the current lang does not match any of
   // the INI files.

   iniPathname = GetPathname();

   TIniFile* iniFile = new TIniFile(iniPathname);

   // Main.
   m_Title = iniFile->ReadString("Main", "Title", "<<UNKNOWN>>");
   m_Icon = iniFile->ReadString("Main", "Icon", "");
   m_Background = iniFile->ReadString("Main", "Background", "");
   m_Width = iniFile->ReadInteger("Main", "Width", 320);
   m_Height = iniFile->ReadInteger("Main", "Height", 240);
   m_StartSound = iniFile->ReadString("Main", "StartSound", "");
   m_ExitSound = iniFile->ReadString("Main", "ExitSound", "");

   // Shell.
   m_LeftMargin = iniFile->ReadInteger("Shell", "LeftMargin", 0);
   m_TopMargin = iniFile->ReadInteger("Shell", "TopMargin", 0);
   m_Spacing = iniFile->ReadInteger("Shell", "Spacing", 0);
   m_ActiveColor = iniFile->ReadInteger("Shell", "ActiveColor",
      255 /* red */);

   m_InactiveColor = iniFile->ReadInteger("Shell", "InactiveColor",
       0 /* black */);

   m_ActiveSound = iniFile->ReadString("Shell", "ActiveSound", "");
   m_ShellSound = iniFile->ReadString("Shell", "ShellSound", "");
   m_FontName = iniFile->ReadString("Shell", "FontName", "Arial");
   m_FontSize = iniFile->ReadInteger("Shell", "FontSize", 18);

   // Play the start sound.
   PlaySound(m_StartSound.c_str(), NULL, SND_FILENAME | SND_ASYNC);

   frmMainframe->Caption = m_Title;
   Application->Title = m_Title;

   if(m_Icon != "")
   {
      Application->Icon->LoadFromFile(m_Icon);
   }

   if(m_Background != "")
   {
      frmMainframe->imgBackground->Picture->LoadFromFile(
         m_Background);
   }

   frmMainframe->ClientWidth = m_Width;
   frmMainframe->ClientHeight = m_Height;

   m_Count = iniFile->ReadInteger("Shell", "Count", 0);
   m_Align = iniFile->ReadInteger("Shell", "Align", 0);

   // Now create the labels.
   for(int i = 1; i <= m_Count; i++)
   {
      TActiveLabel* label = new TActiveLabel(frmMainframe);

      label->Parent = frmMainframe;

      label->Left = m_LeftMargin;
      label->Width = 100;
      label->Height = 32;
      label->ActiveColor = TColor(m_ActiveColor);
      label->InactiveColor = TColor(m_InactiveColor);

      label->EnterSound = m_ActiveSound;
      label->CommandSound = m_ShellSound;

      label->Font->Name = m_FontName;
      label->Font->Size = m_FontSize;

      AnsiString key;
      AnsiString value;

      key = Format("Shell%dTitle", ARRAYOFCONST((i)));
      value = iniFile->ReadString("Shell", key, "<<UNKNOWN>>");
      label->Caption = value;

      label->Top = m_TopMargin +
         ((i - 1) * (label->Height + m_Spacing));


      key = Format("Shell%dCmd", ARRAYOFCONST((i)));
      value = iniFile->ReadString("Shell", key, "");
      label->Command = value;
   }
}

void __fastcall TCDIni::OnExit()
{
   if(m_ExitSound != "")
   {
      PlaySound(m_ExitSound.c_str(), NULL, SND_FILENAME | SND_SYNC);
   }
}

AnsiString __fastcall TCDIni::GetPathname()
{
   // The default INI file is always called CD00.INI. Specific
   // language INI files are named CD<LANGUAGE_CODE>.INI where
   // LANGUAGE_CODE is the two character hexadecimal PRIMARYLANGID
   // value.

   AnsiString pathname;
   TCHAR code[0x100];
   LANGID langId;

   langId = ::GetUserDefaultLangID();

   wsprintf(code, "%02X", PRIMARYLANGID(langId));

   pathname = ".\\DATA\\CD" + AnsiString(code) + ".INI";

   // Now confirm the file exists. If it does not use the default.
   if(!(FileExists(pathname)))
   {
      return(AnsiString(".\\DATA\\CD00.INI"));
   }

   return(pathname);
}
