// glp.cpp

#include "LongPath.h"
#include <assert.h>
#include <stddef.h>


bool isWindowsNT( void ) {

    OSVERSIONINFO osvi = { sizeof osvi };
    assert( 0 == offsetof( OSVERSIONINFO, dwOSVersionInfoSize ) );

    return GetVersionEx( &osvi ) && VER_PLATFORM_WIN32_NT == osvi.dwPlatformId;
}


void comFree( LPVOID p ) {

    IMalloc *pMalloc = 0;
    if ( SUCCEEDED( SHGetMalloc( &pMalloc ) ) ) {
        pMalloc->Free( p );
        pMalloc->Release();
    }
}


typedef BOOL (WINAPI *GETPATHFROMIDLISTW)( LPCITEMIDLIST, LPWSTR );


HRESULT getPathFromIDList( LPCITEMIDLIST pidl, LPTSTR pszPath ) {

    HRESULT hres = ERROR_CALL_NOT_IMPLEMENTED;

#ifdef UNICODE

    if ( isWindowsNT() ) {
        GETPATHFROMIDLISTW fGetPathFromIDListW = 
            (GETPATHFROMIDLISTW) GetProcAddress( 
                GetModuleHandle( _T( "SHELL32" ) ), 
                "SHGetPathFromIDListW" );
        if ( 0 != fGetPathFromIDListW ) {
            hres = fGetPathFromIDListW( pidl, pszPath );
        }
    } else {
        char szLong[ MAX_PATH ] = { 0 };
        const HRESULT hres = SHGetPathFromIDListA( pidl, szLong );
        MultiByteToWideChar( CP_ACP, 0, 
            szLong, -1, pszPath, MAX_PATH );
    }

#else

    hres = SHGetPathFromIDListA( pidl, pszPath );

#endif

    return hres;
}


bool getLongPathName( LPCTSTR pszShort, LPTSTR pszLong ) {

    IShellFolder *pShellFolder = 0;
    HRESULT hres = SHGetDesktopFolder( &pShellFolder );
    if ( SUCCEEDED( hres ) ) {

#ifdef UNICODE
#define wszShort const_cast< LPWSTR >( pszShort )
#else
        WCHAR wszShort[ MAX_PATH ] = { 0 };
        MultiByteToWideChar( CP_ACP, 0, 
            pszShort, -1, wszShort, MAX_PATH );
#endif

        ULONG ulEaten = 0;
        ITEMIDLIST *pidl = 0;
        ULONG ulAttributes = 0;
        hres = pShellFolder->ParseDisplayName( HWND_DESKTOP, 0,
            wszShort, &ulEaten, &pidl, &ulAttributes );
        if ( SUCCEEDED( hres ) ) {
            hres = getPathFromIDList( pidl, pszLong );
            comFree( pidl );
        }
        pShellFolder->Release();
    }

    return SUCCEEDED( hres );
}

// end of file
